#include "../include/QtQuickControlsApplication.h"
#include "../include/SortFilterProxyModel.h"
#include <QtQml/qqmlapplicationengine.h>
#include <QtGui/qsurfaceformat.h>
#include <QtQml/qqml.h>
#include <QtQml/QQmlContext>
#include <include/MessageModel.h>
#include "NetworkingLib/Networking.h"
#include "MessageLib/Client.h"
#include <chrono>
#include <iostream>
#include <include/MessageProducer.h>

Q_DECLARE_METATYPE(message::Message);

int main(int argc, char * argv[])
{
    using namespace std::chrono_literals;

    qmlRegisterType<SortFilterProxyModel>("car.pcgui", 1, 0, "SortFilterProxyModel");
    qmlRegisterType<MessageModel>("car.pcgui", 1, 0, "MessageModel");
    qRegisterMetaType<message::Message>();
    qmlRegisterUncreatableType<MessageList>(
        "car.pcgui", 1, 0, "MessageList", QStringLiteral("MessageList should not be created in QML"));

    networking::Networking net;
    auto messageClient = message::Client::create(net, 10207);
    MessageList messages;
    MessageProducer producer;
    QObject::connect(&producer, SIGNAL(messageProduced(message::Message)),
                     &messages, SLOT(appendMessage(const message::Message &)));

    QtQuickControlsApplication app(argc, argv);
    if (QCoreApplication::arguments().contains(QLatin1String("--coreprofile")))
    {
        QSurfaceFormat fmt;
        fmt.setVersion(4, 4);
        fmt.setProfile(QSurfaceFormat::CoreProfile);
        QSurfaceFormat::setDefaultFormat(fmt);
    }

    QQmlApplicationEngine engine{};
    engine.rootContext()->setContextProperty(QStringLiteral("messages"), &messages);
    engine.load(QUrl("qrc:/MessageGui.qml"));
    if (engine.rootObjects().isEmpty())
        return -1;

    messageClient->requestMessagesPeriodically(
        "127.0.0.1", 10ms, 3s,
        [&producer](const auto & message)
        { producer.produceMessage(message); });

    return app.exec();
}
